@@ -1,6 +1,9 @@
-from typing import Dict, Optional
+from functools import lru_cache
 import re
+import sys
+from typing import Dict, Optional, List, Tuple
 
+from robocorp_ls_core.lsp import DiagnosticSeverity, DiagnosticTag
 from robocorp_ls_core.protocols import check_implements
 from robocorp_ls_core.robotframework_log import get_logger
 from robotframework_ls.impl.ast_utils import MAX_ERRORS
@@ -12,13 +15,13 @@ from robotframework_ls.impl.protocols import (
     INode,
     IVariableFound,
     AbstractVariablesCollector,
+    VariableKind,
 )
-from robocorp_ls_core.lsp import DiagnosticSeverity, DiagnosticTag
 from robotframework_ls.impl.robot_lsp_constants import (
     OPTION_ROBOT_LINT_VARIABLES,
     OPTION_ROBOT_LINT_IGNORE_VARIABLES,
+    OPTION_ROBOT_LINT_IGNORE_ENVIRONMENT_VARIABLES,
 )
-from functools import lru_cache
 
 
 log = get_logger(__name__)
@@ -57,33 +60,58 @@ class _KeywordContainer(object):
 
 class _VariablesCollector(AbstractVariablesCollector):
     def __init__(self, on_unresolved_variable_import):
-        self._variables_collected = set()
-        self._template_variables_collected = []
+        self._variables_collected: Dict[str, List[IVariableFound]] = {}
+        self._template_variables_collected: List[Tuple[str, IVariableFound]] = []
         self.on_unresolved_variable_import = on_unresolved_variable_import
 
-    def accepts(self, variable_name: str) -> bool:
-        from robotframework_ls.impl.text_utilities import normalize_robot_name
+        self._env_variables_collected: Dict[str, IVariableFound] = {}
 
-        normalized = normalize_robot_name(variable_name)
-        if "{" in normalized:
-            self._template_variables_collected.append(normalized)
-        self._variables_collected.add(normalized)
-        # We don't want to create the IVariableFound, just the names should be
-        # enough for our usage.
-        return False
+    def accepts(self, variable_name: str) -> bool:
+        return True
+
+    def on_env_variable(self, variable_found: IVariableFound):
+        upper = variable_found.variable_name.upper()
+        self._env_variables_collected[upper] = variable_found
 
     def on_variable(self, variable_found: IVariableFound):
-        pass
+        from robotframework_ls.impl.text_utilities import normalize_robot_name
 
-    def contains_variable(self, variable_name):
+        normalized = normalize_robot_name(variable_found.variable_name)
+        if "{" in normalized:
+            self._template_variables_collected.append((normalized, variable_found))
+
+        lst = self._variables_collected.get(normalized)
+        if lst is None:
+            lst = self._variables_collected[normalized] = []
+        lst.append(variable_found)
+
+    def contains_env_variable(self, variable_name_upper: str) -> bool:
+        return variable_name_upper in self._env_variables_collected
+
+    def contains_variable(
+        self, variable_name: str, var_line: int, var_col_offset: int
+    ) -> bool:
         from robotframework_ls.impl.text_utilities import matches_name_with_variables
 
-        if variable_name in self._variables_collected:
-            return True
+        variables_found: Optional[List[IVariableFound]] = self._variables_collected.get(
+            variable_name
+        )
+        if variables_found:
+            for v in variables_found:
+                if v.lineno < var_line:
+                    return True
 
-        for template_var in self._template_variables_collected:
-            if matches_name_with_variables(variable_name, template_var):
-                return True
+                if (
+                    v.lineno == var_line
+                    and v.variable_kind == VariableKind.ARGUMENT
+                    and v.col_offset < var_col_offset
+                ):
+                    return True
+
+        for template_var, v in self._template_variables_collected:
+            if v.lineno < var_line:
+                if matches_name_with_variables(variable_name, template_var):
+                    return True
 
         return False
 
@@ -394,66 +422,20 @@ def collect_analysis_errors(initial_completion_context):
     return errors
 
 
-def _is_number_var(normalized_variable_name):
-    # see: robot.variables.finders.NumberFinder
-    try:
-        bases = {"0b": 2, "0o": 8, "0x": 16}
-        if normalized_variable_name.startswith(tuple(bases)):
-            return int(
-                normalized_variable_name[2:], bases[normalized_variable_name[:2]]
-            )
-        int(normalized_variable_name)
-        return True
-    except:
-        pass  # Let's try float...
-
-    try:
-        float(normalized_variable_name)
-        return True
-    except:
-        pass
-
-    return False
-
-
-def _is_python_eval_var(normalized_variable_name):
-    return (
-        len(normalized_variable_name) >= 2
-        and normalized_variable_name[0] == "{"
-        and normalized_variable_name[-1] == "}"
-    )
-
-
 @lru_cache(maxsize=1000)
 def _skip_variable_analysis(normalized_variable_name):
+    from robotframework_ls.impl.variable_resolve import is_number_var
+    from robotframework_ls.impl.variable_resolve import is_python_eval_var
 
-    if _is_number_var(normalized_variable_name):
+    if is_number_var(normalized_variable_name):
         return True
 
-    if _is_python_eval_var(normalized_variable_name):
+    if is_python_eval_var(normalized_variable_name):
         return True
 
     return False
 
 
-_match_extended = re.compile(
-    r"""
-    (.+?)          # base name (group 1)
-    ([^\s\w].+)    # extended part (group 2)
-""",
-    re.UNICODE | re.VERBOSE,
-).match
-
-
-def _extract_base_from_extended_var_name(normalized_variable_name):
-    m = _match_extended(normalized_variable_name)
-    if m is None:
-        return normalized_variable_name
-
-    base_name, _extended = m.groups()
-    return base_name
-
-
 def _env_vars_upper():
     import os
 
@@ -464,6 +446,7 @@ def _collect_undefined_variables_errors(initial_completion_context):
     from robotframework_ls.impl import ast_utils
     from robotframework_ls.impl.variable_resolve import normalize_variable_name
     from robotframework_ls.impl.ast_utils import create_error_from_node
+    from robotframework_ls.impl.variable_resolve import robot_search_variable
 
     config = initial_completion_context.config
     if config is not None and not config.get_setting(
@@ -511,6 +494,15 @@ def _collect_undefined_variables_errors(initial_completion_context):
             for x in config.get_setting(OPTION_ROBOT_LINT_IGNORE_VARIABLES, list, [])
         )
 
+    ignore_environment_variables = set()
+    if config is not None:
+        ignore_environment_variables.update(
+            str(x).upper()
+            for x in config.get_setting(
+                OPTION_ROBOT_LINT_IGNORE_ENVIRONMENT_VARIABLES, list, []
+            )
+        )
+
     ast = initial_completion_context.get_ast()
 
     globals_collector = _VariablesCollector(
@@ -527,6 +519,7 @@ def _collect_undefined_variables_errors(initial_completion_context):
     env_vars_upper = None
 
     for token_info in ast_utils.iter_variable_references(ast):
+
         initial_completion_context.check_cancelled()
 
         if token_info.node.__class__.__name__ in (
@@ -538,18 +531,37 @@ def _collect_undefined_variables_errors(initial_completion_context):
             # import.
             continue
 
-        var_name = token_info.token.value
-        if token_info.var_identifier == "%":
-            from robotframework_ls.impl.variable_resolve import extract_variable_base
+        if (
+            token_info.node.__class__.__name__ == "KeywordCall"
+            and token_info.node.keyword == "Comment"
+        ):
+            # Special handling for 'Comment' keyword (variables are not
+            # resolved when calling the 'Comment' keyword).
+            # https://github.com/robocorp/robotframework-lsp/issues/665
+            continue
 
-            if "=" in var_name:
-                # Consider case: %{SOME_VAR=defaultvalue}
+        var_name = token_info.token.value
+        var_line = token_info.token.lineno - 1  # We want it 0-based
+        var_col_offset = token_info.token.col_offset
+
+        if token_info.var_info.var_identifier == "%":
+
+            if "=" in var_name + token_info.var_info.extended_part:
+                # Consider case: %{SOME_VAR=}
+                # Consider case: %{SOME_VAR=default val}
+                continue
+
+            var_name_upper = var_name.upper()
+            if var_name_upper in ignore_environment_variables:
                 continue
 
             if env_vars_upper is None:
                 env_vars_upper = _env_vars_upper()
 
-            if extract_variable_base(var_name).upper() not in env_vars_upper:
+            if (
+                var_name_upper not in env_vars_upper
+                and not globals_collector.contains_env_variable(var_name_upper)
+            ):
                 # Environment variable
                 yield create_error_from_node(
                     token_info.node,
@@ -558,10 +570,23 @@ def _collect_undefined_variables_errors(initial_completion_context):
                 )
             continue
 
-        normalized_variable_name = normalize_variable_name(var_name)
+        check_names = [normalize_variable_name(var_name)]
+        if token_info.var_info.extended_part.strip():
+
+            robot_match_in_ext = robot_search_variable(
+                token_info.var_info.extended_part
+            )
+            if robot_match_in_ext is not None and robot_match_in_ext.base:
+                continue
+
+            check_names.append(
+                normalize_variable_name(var_name + token_info.var_info.extended_part)
+            )
+
+        locals_collector = None
 
         found = False
-        while not found:
+        for normalized_variable_name in check_names:
             if normalized_variable_name in ignore_variables:
                 found = True
                 break
@@ -570,31 +595,29 @@ def _collect_undefined_variables_errors(initial_completion_context):
                 found = True
                 break
 
-            if globals_collector.contains_variable(normalized_variable_name):
+            if globals_collector.contains_variable(
+                normalized_variable_name, sys.maxsize, 0
+            ):
                 found = True
                 break
 
-            locals_collector = _VariablesCollector(lambda *args, **kwargs: None)
-            local_ctx = initial_completion_context.create_copy_with_selection(
-                line=token_info.token.lineno - 1,
-                col=token_info.token.col_offset - 1,
-            )
+            if locals_collector is None:
+                locals_collector = _VariablesCollector(lambda *args, **kwargs: None)
+                local_ctx = initial_completion_context.create_copy_with_selection(
+                    line=token_info.token.lineno - 1,
+                    col=token_info.token.col_offset,
+                )
 
-            variable_completions.collect_local_variables(
-                local_ctx, locals_collector, token_info
-            )
-            if locals_collector.contains_variable(normalized_variable_name):
+                variable_completions.collect_local_variables(
+                    local_ctx, locals_collector, token_info
+                )
+
+            if locals_collector.contains_variable(
+                normalized_variable_name, var_line, var_col_offset
+            ):
                 found = True
                 break
 
-            extracted_base = _extract_base_from_extended_var_name(
-                normalized_variable_name
-            )
-            if extracted_base and extracted_base != normalized_variable_name:
-                normalized_variable_name = extracted_base
-            else:
-                break
-
         if not found:
             yield create_error_from_node(
                 token_info.node,
