@@ -10,8 +10,10 @@ from robotframework_ls.impl.protocols import (
     IRobotToken,
     IVariableFound,
     AbstractVariablesCollector,
+    VarTokenInfo,
+    IVariableDefinition,
 )
-from robocorp_ls_core.protocols import check_implements, IDocumentSelection
+from robocorp_ls_core.protocols import check_implements
 from typing import Optional, Sequence, List
 from robocorp_ls_core.lsp import RangeTypedDict, MarkupContentTypedDict, MarkupKind
 from robocorp_ls_core.basic import isinstance_name
@@ -272,17 +274,53 @@ class _FindDefinitionKeywordCollector(AbstractKeywordCollector):
 
 class _FindDefinitionVariablesCollector(AbstractVariablesCollector):
     def __init__(
-        self, sel: IDocumentSelection, token: IRobotToken, robot_string_matcher
+        self, completion_context: ICompletionContext, var_token_info: VarTokenInfo
     ):
-        self.matches: List[IDefinition] = []
-        self.sel = sel
-        self.token = token
-        self.matcher = robot_string_matcher
+        from robotframework_ls.impl.string_matcher import RobotStringMatcher
+
+        self.matches: List[IVariableDefinition] = []
+        self.var_token_info = var_token_info
+
+        token: IRobotToken = var_token_info.token
+        value = token.value
+        self.stack = var_token_info.stack
+        self._matcher = RobotStringMatcher(value)
+
+        self._matcher_with_extended_part = None
+        if var_token_info.var_info.extended_part.strip():
+            full_name = value + var_token_info.var_info.extended_part
+            self._matcher_with_extended_part = RobotStringMatcher(full_name)
+
+        self._completion_context = completion_context
 
     def accepts(self, variable_name: str) -> bool:
-        return self.matcher.is_variable_name_match(variable_name)
+        if self._matcher.is_variable_name_match(variable_name):
+            return True
+        if self._matcher_with_extended_part is not None:
+            return self._matcher_with_extended_part.is_variable_name_match(
+                variable_name
+            )
+        return False
 
     def on_variable(self, variable_found: IVariableFound):
+        from robotframework_ls.impl.ast_utils import matches_stack
+
+        is_local_variable = variable_found.is_local_variable
+        if is_local_variable:
+            if self._completion_context.doc.path != variable_found.source:
+                return
+
+            if not matches_stack(self.stack, variable_found.stack):
+                return
+
+            if variable_found.lineno > self.var_token_info.token.lineno - 1:
+                return
+
+            if variable_found.lineno == self.var_token_info.token.lineno - 1:
+                # We need to check the column too.
+                if variable_found.col_offset > self.var_token_info.token.col_offset:
+                    return
+
         definition = _DefinitionFromVariable(variable_found)
         self.matches.append(definition)
 
@@ -290,6 +328,20 @@ class _FindDefinitionVariablesCollector(AbstractVariablesCollector):
         _: IVariablesCollector = check_implements(self)
 
 
+def find_variable_definition(
+    completion_context: ICompletionContext, var_token_info: VarTokenInfo
+) -> Optional[Sequence[IVariableDefinition]]:
+    from robotframework_ls.impl.variable_completions import collect_variables
+
+    token = var_token_info.token
+    completion_context = completion_context.create_copy_with_selection(
+        line=token.lineno - 1, col=token.col_offset
+    )
+    collector = _FindDefinitionVariablesCollector(completion_context, var_token_info)
+    collect_variables(completion_context, collector)
+    return collector.matches
+
+
 def find_keyword_definition(
     completion_context: ICompletionContext, token_info: TokenInfo
 ) -> Optional[Sequence[IKeywordDefinition]]:
@@ -303,7 +355,10 @@ def find_keyword_definition(
     from robotframework_ls.impl import ast_utils
 
     token = ast_utils.get_keyword_name_token(
-        token_info.stack, token_info.node, token_info.token
+        token_info.stack,
+        token_info.node,
+        token_info.token,
+        accept_only_over_keyword_name=False,
     )
     if token is None:
         if token_info.token.type == token_info.token.KEYWORD_NAME:
@@ -343,8 +398,6 @@ def find_definition_extended(
         at this place (callers are responsible for validating entries).
     """
     from robotframework_ls.impl import ast_utils
-    from robotframework_ls.impl.string_matcher import RobotStringMatcher
-    from robotframework_ls.impl.variable_completions import collect_variables
     from robotframework_ls.impl.libspec_manager import LibspecManager
 
     token_info = completion_context.get_current_token()
@@ -408,14 +461,10 @@ def find_definition_extended(
 
     var_token_info = completion_context.get_current_variable()
     if var_token_info is not None:
-        token = var_token_info.token
-        value = token.value
-        collector = _FindDefinitionVariablesCollector(
-            completion_context.sel, token, RobotStringMatcher(value)
-        )
-        collect_variables(completion_context, collector)
-        return _DefinitionInfo(
-            collector.matches, ast_utils.create_range_from_token(token)
-        )
+        var_matches = find_variable_definition(completion_context, var_token_info)
+        if var_matches:
+            return _DefinitionInfo(
+                var_matches, ast_utils.create_range_from_token(var_token_info.token)
+            )
 
     return None
