@@ -6,6 +6,7 @@ from typing import Optional, List, Deque, Iterator, Dict, Union, Sequence
 import itertools
 from robocorp_ls_core.lsp import Error
 from robocorp_ls_core.constants import Null, NULL
+import typing
 
 
 class _Match:
@@ -169,6 +170,7 @@ class KeywordArgumentAnalysis:
         from collections import deque
         from robotframework_ls.impl.text_utilities import is_variable_text
         from robotframework_ls.impl.variable_resolve import find_split_index
+        from robotframework_ls.impl.variable_resolve import has_variable
 
         # Pre-requisite.
         keyword_token = usage_info.get_token_to_report_argument_missing()
@@ -206,6 +208,12 @@ class KeywordArgumentAnalysis:
 
             definition_keyword_name_to_arg[arg_name] = definition_arg
 
+        # The keys from definition_keyword_name_to_arg but without mutating
+        # it during the analysis.
+        all_definition_keyword_names: typing.FrozenSet[str] = frozenset(
+            definition_keyword_name_to_arg.keys()
+        )
+
         tokens_args_to_iterate = self._iter_args(usage_info.node.tokens)
         # Fill positional args
         for token_arg in tokens_args_to_iterate:
@@ -264,6 +272,11 @@ class KeywordArgumentAnalysis:
                     break
 
                 name = token_arg.value[:eq_index]
+
+                if has_variable(name):
+                    # If an argument has variables, skip the analysis.
+                    return
+
                 found_definition_arg = definition_keyword_name_to_arg.get(name, None)
                 if found_definition_arg is not None:
                     if definition_arg_matched.get(found_definition_arg):
@@ -317,6 +330,10 @@ class KeywordArgumentAnalysis:
             eq_index = find_split_index(token_arg.value)
             if eq_index >= 0:
                 name = token_arg.value[:eq_index]
+                if has_variable(name):
+                    # If an argument has variables, skip the analysis.
+                    return
+
                 found_definition_arg = definition_keyword_name_to_arg.pop(name, None)
                 if not found_definition_arg:
                     if self.found_keyword_arg is not None:
@@ -325,12 +342,20 @@ class KeywordArgumentAnalysis:
                         )
                     else:
                         if collect_errors:
-                            error = create_error_from_node(
-                                usage_info.node,
-                                f"Unexpected named argument: {name}",
-                                tokens=[token_arg],
-                            )
-                            yield error
+                            if name in all_definition_keyword_names:
+                                error = create_error_from_node(
+                                    usage_info.node,
+                                    f"Argument already specified previously: {name}",
+                                    tokens=[token_arg],
+                                )
+                                yield error
+                            else:
+                                error = create_error_from_node(
+                                    usage_info.node,
+                                    f"Unexpected named argument: {name}",
+                                    tokens=[token_arg],
+                                )
+                                yield error
 
                 else:
                     usage_token_id_to_definition_arg_match[id(token_arg)] = _Match(
