@@ -10,7 +10,6 @@ from typing import (
     Generic,
     Iterator,
     Callable,
-    Union,
     Hashable,
     Dict,
 )
@@ -31,6 +30,7 @@ from robocorp_ls_core.lsp import (
     RangeTypedDict,
     MarkupContentTypedDict,
     CompletionItemTypedDict,
+    LSPMessages,
 )
 import typing
 from robocorp_ls_core.ordered_set import OrderedSet
@@ -116,12 +116,12 @@ class IRobotToken(Protocol):
 
     type: str
     value: str
-    lineno: int
-    col_offset: int
+    lineno: int  # 1-based
+    col_offset: int  # 0-based
     error: Any
 
     @property
-    def end_col_offset(self) -> int:
+    def end_col_offset(self) -> int:  # 0-based
         pass
 
     def tokenize_variables(self) -> Iterator["IRobotToken"]:
@@ -215,33 +215,57 @@ class TokenInfo:
     __repr__ = __str__
 
 
-class VarTokenInfo:
-
-    __slots__ = ["stack", "node", "token", "var_identifier", "context"]
+class AdditionalVarInfo:
 
     CONTEXT_UNDEFINED = 0
     CONTEXT_EXPRESSION = 1
 
+    __slots__ = ["var_identifier", "context", "extended_part"]
+
+    def __init__(
+        self,
+        var_identifier: str = "",
+        context: int = CONTEXT_UNDEFINED,
+        extended_part: str = "",
+    ):
+        """
+        :param var_identifier: One of: $,@,%
+        """
+
+        self.var_identifier = var_identifier
+        self.context = context
+        self.extended_part = extended_part
+
+    def __str__(self):
+        info = [f"AdditionalVarInfo({self.var_identifier}"]
+        if self.context:
+            info.append(f" -- ctx: {self.context}")
+        if self.extended_part:
+            info.append(f" -- extended: {self.extended_part}")
+        info.append(")")
+        return "".join(info)
+
+    __repr__ = __str__
+
+
+class VarTokenInfo:
+
+    __slots__ = ["stack", "node", "token", "var_info"]
+
     def __init__(
         self,
         stack: Tuple[INode, ...],
         node: INode,
         token: IRobotToken,
-        var_identifier: str,
-        context: int = CONTEXT_UNDEFINED,
+        var_info: AdditionalVarInfo,
     ):
-        """
-        :param var_identifier: One of: $,@,%
-        :param context:
-        """
         self.stack = stack
         self.node = node
         self.token = token
-        self.var_identifier = var_identifier
-        self.context = context
+        self.var_info = var_info
 
     def __str__(self):
-        return f"VarTokenInfo({self.token.value} -- id: {self.var_identifier} -- in: {self.node.__class__.__name__})"
+        return f"VarTokenInfo({self.token.value} (in {self.node.__class__.__name__}) - {self.var_info})"
 
     __repr__ = __str__
 
@@ -668,12 +692,25 @@ class IKeywordDefinition(IDefinition, Protocol):
     keyword_found: IKeywordFound
 
 
+class IVariableDefinition(IDefinition, Protocol):
+
+    variable_found: "IVariableFound"
+
+
 def cast_to_keyword_definition(definition: IDefinition) -> Optional[IKeywordDefinition]:
     if hasattr(definition, "keyword_found"):
         return typing.cast(IKeywordDefinition, definition)
     return None
 
 
+def cast_to_variable_definition(
+    definition: IDefinition,
+) -> Optional[IVariableDefinition]:
+    if hasattr(definition, "variable_found"):
+        return typing.cast(IVariableDefinition, definition)
+    return None
+
+
 class IBaseCompletionContext(Protocol):
     @property
     def monitor(self) -> Optional[IMonitor]:
@@ -820,6 +857,12 @@ class ICompletionContext(Protocol):
     ) -> None:
         pass
 
+    @property
+    def lsp_messages(
+        self,
+    ) -> Optional[LSPMessages]:
+        pass
+
     @property
     def variables_from_arguments_files_loader(
         self,
@@ -998,6 +1041,20 @@ class VariableKind:
     PYTHON = "Variable (python)"
     YAML = "Variable (yaml)"
     ARGUMENTS_FILE = "Arguments file"
+    LOCAL_ASSIGN_VARIABLE = "Variable (local assign)"
+    LOCAL_SET_VARIABLE = "Variable (local set)"
+    TASK_SET_VARIABLE = "Variable (task set)"
+    TEST_SET_VARIABLE = "Variable (test set)"
+    SUITE_SET_VARIABLE = "Variable (suite set)"
+    GLOBAL_SET_VARIABLE = "Variable (global)"
+    ENV_SET_VARIABLE = "Variable (environment)"
+
+
+LOCAL_ASSIGNS_VARIABLE_KIND = {
+    VariableKind.ARGUMENT,
+    VariableKind.LOCAL_ASSIGN_VARIABLE,
+    VariableKind.LOCAL_SET_VARIABLE,
+}
 
 
 class IVariableFound(Protocol):
@@ -1019,12 +1076,22 @@ class IVariableFound(Protocol):
 
     :ivar lineno:
         Line where it was found (0-based).
+
+    :ivar stack:
+        The stack where the variable was found (only available if it was
+        found in a robot file where the ast is available -- i.e.: settings,
+        yaml, python, etc. variables don't have a stack available).
     """
 
     variable_name: str = ""
     variable_value: str = ""
     variable_kind: str = VariableKind.VARIABLE
     completion_context: Optional[ICompletionContext] = None
+    stack: Optional[Tuple[INode, ...]] = None
+
+    @property
+    def is_local_variable(self) -> bool:
+        pass
 
     @property
     def source(self) -> str:
@@ -1070,8 +1137,19 @@ class IVariablesCollector(Protocol):
     ):
         pass
 
+    def on_env_variable(self, variable_found: IVariableFound):
+        """
+        Called for environment variables using the
+        'Set Keyword Variable' keyword.
+
+        Note: doesn't call `accepts` first.
+        """
+
 
 class AbstractVariablesCollector:
+    def on_env_variable(self, variable_found: IVariableFound):
+        pass
+
     def on_unresolved_variable_import(
         self,
         completion_context: "ICompletionContext",
