@@ -159,6 +159,9 @@ RF_TOKEN_TYPE_TO_TOKEN_TYPE_INDEX = {
     ERROR: TOKEN_TYPE_TO_INDEX["error"],
     FATAL_ERROR: TOKEN_TYPE_TO_INDEX["error"],
     OPTION: TOKEN_TYPE_TO_INDEX["argumentValue"],
+    # Special: we don't have a token regarding such operators in RF, so, add
+    # 'variableOperator' as being valid for both.
+    "variableOperator": TOKEN_TYPE_TO_INDEX["variableOperator"],
 }
 
 
@@ -184,6 +187,7 @@ VARIABLE_OPERATOR_INDEX = TOKEN_TYPE_TO_INDEX["variableOperator"]
 SETTING_INDEX = TOKEN_TYPE_TO_INDEX["setting"]
 PARAMETER_NAME_INDEX = TOKEN_TYPE_TO_INDEX["parameterName"]
 DOCUMENTATION_INDEX = TOKEN_TYPE_TO_INDEX["documentation"]
+CONTROL_INDEX = TOKEN_TYPE_TO_INDEX["control"]
 
 
 def _tokenize_changing_argument_to_type(tokenize_variables_generator, use_type):
@@ -246,27 +250,28 @@ def _tokenize_token(
 
     # Step 1: cast to KEYWORD if needed.
     if use_token_type == ARGUMENT:
-        in_documentation = node.__class__.__name__ == "Documentation"
+        in_documentation = node.__class__.__name__ == "Documentation" or (
+            node.__class__.__name__ == "KeywordCall" and node.keyword == "Comment"
+        )
 
         if not in_documentation:
 
             in_expression = is_node_with_expression_argument(node)
             if scope.args_as_keywords_handler is not None:
-                if scope.args_as_keywords_handler.consider_current_argument_token_as_keyword(
-                    use_token
-                ):
+                tok_type = scope.args_as_keywords_handler.next_tok_type(use_token)
+                if tok_type == scope.args_as_keywords_handler.KEYWORD:
                     use_token_type = KEYWORD
 
-                if not in_expression:
-                    in_expression = (
-                        scope.args_as_keywords_handler.was_last_expression_argument
-                    )
+                elif tok_type == scope.args_as_keywords_handler.EXPRESSION:
+                    in_expression = True
+
+                elif tok_type == scope.args_as_keywords_handler.CONTROL:
+                    yield use_token, CONTROL_INDEX
+                    return
 
             if in_expression:
-                for token in iter_expression_tokens(use_token, "argumentValue"):
-                    token_type_index = scope.get_index_from_internal_token_type(
-                        token.type
-                    )
+                for token, _var_info in iter_expression_tokens(use_token):
+                    token_type_index = scope.get_index_from_rf_token_type(token.type)
                     if token_type_index is not None:
                         yield token, token_type_index
                 return
@@ -370,46 +375,16 @@ def _tokenize_subvars(var_token, token_type_index, scope):
         yield var_token, token_type_index
         return
 
-    if var_token.value.startswith("{") and var_token.value.endswith("}"):
-        from robotframework_ls.impl.ast_utils import split_token_in_3
-        from robotframework_ls.impl.ast_utils import iter_expression_tokens
+    from robotframework_ls.impl import ast_utils
 
-        # i.e.: We're dealing with an expression.
-        first, second, third = split_token_in_3(
-            var_token,
-            "variableOperator",
-            "argumentValue",
-            "variableOperator",
-            1,
-            -1,
-        )
+    var_type = var_token.type
+    if var_type == "variableOperator":
+        var_type = var_token.VARIABLE
 
-        yield first, VARIABLE_OPERATOR_INDEX
-
-        for token in iter_expression_tokens(second, "argumentValue"):
-            token_type_index = scope.get_index_from_internal_token_type(token.type)
-            if token_type_index is not None:
-                yield token, token_type_index
-
-        yield third, VARIABLE_OPERATOR_INDEX
-        return
-
-    from robotframework_ls.impl.ast_utils import RobotMatchTokensGenerator
-
-    robot_match_generator = RobotMatchTokensGenerator(var_token, "variable")
-    from robotframework_ls.impl.variable_resolve import iter_robot_variable_matches
-
-    for robot_match, relative_index in iter_robot_variable_matches(var_token.value):
-        for token in robot_match_generator.gen_tokens_from_robot_match(
-            robot_match,
-            relative_index,
-        ):
-            token_type_index = scope.get_index_from_internal_token_type(token.type)
-            if token_type_index is not None:
-                yield token, token_type_index
-
-    for token in robot_match_generator.gen_default_type(len(var_token.value)):
-        token_type_index = scope.get_index_from_internal_token_type(token.type)
+    for token, _var_info in ast_utils._tokenize_subvars_tokens(
+        var_token, var_type=var_type
+    ):
+        token_type_index = scope.get_index_from_rf_token_type(token.type)
         if token_type_index is not None:
             yield token, token_type_index
 
