@@ -1,4 +1,4 @@
-from typing import Optional, List, Set
+from typing import Optional, List, Dict, Tuple, Sequence
 
 from robocorp_ls_core.lsp import CompletionItemTypedDict
 from robocorp_ls_core.protocols import check_implements, IDocumentSelection
@@ -13,12 +13,15 @@ from robotframework_ls.impl.protocols import (
     AbstractVariablesCollector,
     VariableKind,
     VarTokenInfo,
+    INode,
+    AdditionalVarInfo,
 )
 from robotframework_ls.impl.text_utilities import normalize_robot_name
 from robotframework_ls.impl.variable_types import (
     VariableFoundFromToken,
     VariableFoundFromYaml,
 )
+from robotframework_ls.impl.ast_utils import KEYWORD_SET_ENV_TO_VAR_KIND
 
 
 log = get_logger(__name__)
@@ -51,7 +54,7 @@ class _Collector(AbstractVariablesCollector):
 
         label = variable_found.variable_name
 
-        if self.var_token_info.context == self.var_token_info.CONTEXT_EXPRESSION:
+        if self.var_token_info.var_info.context == AdditionalVarInfo.CONTEXT_EXPRESSION:
             # On expressions without '${...}' (just $var_name), we can't use spaces.
             label = label.replace(" ", "_")
 
@@ -94,7 +97,8 @@ def _collect_variables_from_set_keywords(
     ast,
     completion_context: ICompletionContext,
     collector: IVariablesCollector,
-    accept_sets_in: Set[str],
+    accept_sets_in: Dict[str, str],
+    env_vars: bool = False,
 ):
     from robot.api import Token
     from robotframework_ls.impl import ast_utils
@@ -102,7 +106,8 @@ def _collect_variables_from_set_keywords(
     for keyword_usage in ast_utils.iter_keyword_usage_tokens(
         ast, collect_args_as_keywords=True
     ):
-        if normalize_robot_name(keyword_usage.name) in accept_sets_in:
+        var_kind = accept_sets_in.get(normalize_robot_name(keyword_usage.name))
+        if var_kind is not None:
             var_name_tok = None
             var_value_tok = None
             for tok in keyword_usage.node.tokens:
@@ -121,7 +126,7 @@ def _collect_variables_from_set_keywords(
                     var_name = var_name[1:]
                     start_offset += 1
 
-                if var_name.startswith(("@", "$", "&")):
+                if var_name.startswith(("@", "$", "&", "%")):
                     var_name = var_name[1:]
                     start_offset += 1
 
@@ -129,9 +134,9 @@ def _collect_variables_from_set_keywords(
                     var_name = var_name[1:-1]
                     start_offset += 1
 
-                if collector.accepts(var_name):
+                if env_vars or collector.accepts(var_name):
                     base_token = ast_utils.copy_token_replacing(
-                        var_name_tok, col_offset=var_name_tok.col_offset, value=var_name
+                        var_name_tok, col_offset=start_offset, value=var_name
                     )
                     variable_value = ""
                     if var_value_tok is not None:
@@ -142,8 +147,13 @@ def _collect_variables_from_set_keywords(
                         base_token,
                         variable_value,
                         variable_name=var_name,
+                        variable_kind=var_kind,
+                        stack=keyword_usage.stack,
                     )
-                    collector.on_variable(variable_found)
+                    if env_vars:
+                        collector.on_env_variable(variable_found)
+                    else:
+                        collector.on_variable(variable_found)
 
 
 def _collect_current_doc_variables(
@@ -152,6 +162,8 @@ def _collect_current_doc_variables(
     """
     :param CompletionContext completion_context:
     """
+    from robotframework_ls.impl.ast_utils import KEYWORD_SET_GLOBAL_TO_VAR_KIND
+
     # Get keywords defined in the file itself
     completion_context.check_cancelled()
 
@@ -161,15 +173,19 @@ def _collect_current_doc_variables(
         if collector.accepts(variable_found.variable_name):
             collector.on_variable(variable_found)
 
-    accept_sets_in = {
-        normalize_robot_name("Set Task Variable"),
-        normalize_robot_name("Set Test Variable"),
-        normalize_robot_name("Set Suite Variable"),
-        normalize_robot_name("Set Global Variable"),
-    }
+    _collect_variables_from_set_keywords(
+        completion_context.get_ast(),
+        completion_context,
+        collector,
+        KEYWORD_SET_GLOBAL_TO_VAR_KIND,
+    )
 
     _collect_variables_from_set_keywords(
-        completion_context.get_ast(), completion_context, collector, accept_sets_in
+        completion_context.get_ast(),
+        completion_context,
+        collector,
+        KEYWORD_SET_ENV_TO_VAR_KIND,
+        env_vars=True,
     )
 
 
@@ -311,6 +327,11 @@ def _collect_global_variables_from_document_context(
     collector: IVariablesCollector,
     only_current_doc=False,
 ):
+    """
+    :param only_current_doc:
+        If true we won't collect variables from dependencies.
+    """
+
     completion_context.check_cancelled()
     _collect_current_doc_variables(completion_context, collector)
 
@@ -322,14 +343,13 @@ def _collect_global_variables_from_document_context(
 
 def _collect_arguments(
     completion_context: ICompletionContext,
-    node,
+    stack: Tuple[INode, ...],
+    node: INode,
     collector: IVariablesCollector,
 ):
     from robotframework_ls.impl import ast_utils
 
-    for arg_token, _var_identifier in ast_utils.iter_keyword_arguments_as_tokens(
-        node, tokenize_keyword_name=True
-    ):
+    for arg_token in ast_utils.iter_keyword_arguments_as_tokens(node):
         name = arg_token.value
         if collector.accepts(name):
             variable_found = VariableFoundFromToken(
@@ -338,6 +358,7 @@ def _collect_arguments(
                 "",
                 variable_name=name,
                 variable_kind=VariableKind.ARGUMENT,
+                stack=stack,
             )
             collector.on_variable(variable_found)
 
@@ -346,15 +367,25 @@ def collect_variables(
     completion_context: ICompletionContext,
     collector: IVariablesCollector,
     only_current_doc=False,
-):
+) -> None:
     token_info = completion_context.get_current_token()
+    # Note: we'll collect all variables in the current context -- i.e.:
+    # Keyword/Test Case locals + globals -- regardless of whether we're inside a
+    # variable right now (so, we can just use get_current_token() and not
+    # get_current_variable()).
     if token_info is not None:
         collect_local_variables(completion_context, collector, token_info)
 
     collect_global_variables(completion_context, collector, only_current_doc)
 
 
-def _collect_global_static_variables(completion_context, collector):
+def _collect_global_static_variables(
+    completion_context: ICompletionContext,
+    collector: IVariablesCollector,
+) -> None:
+    """
+    Collects variables from builtins and settings.
+    """
     for (
         var_found
     ) in completion_context.get_settings_normalized_var_name_to_var_found().values():
@@ -381,6 +412,10 @@ def collect_global_variables(
     collector: IVariablesCollector,
     only_current_doc=False,
 ):
+    """
+    :param only_current_doc:
+        If true we won't collect variables from dependencies nor builtins/settings.
+    """
     _collect_global_variables_from_document_context(
         completion_context, collector, only_current_doc=only_current_doc
     )
@@ -395,33 +430,34 @@ def collect_local_variables(
     token_info: TokenInfo,
 ):
     from robotframework_ls.impl import ast_utils
+    from robotframework_ls.impl.ast_utils import KEYWORD_SET_LOCAL_TO_VAR_KIND
+    from robotframework_ls.impl.ast_utils import get_local_variable_stack_and_node
+
+    stack: Tuple[INode, ...]
 
     if token_info.stack:
-        for stack_node in reversed(token_info.stack):
-            if stack_node.__class__.__name__ in ("Keyword", "TestCase"):
-                break
-        else:
-            stack_node = token_info.stack[0]
+        stack, stack_node = get_local_variable_stack_and_node(token_info.stack)
     else:
         stack_node = completion_context.get_ast_current_section()
+        stack = (stack_node,)
 
     for assign_node_info in ast_utils.iter_local_assigns(stack_node):
         completion_context.check_cancelled()
         if collector.accepts(assign_node_info.token.value):
             rep = " ".join(tok.value for tok in assign_node_info.node.tokens)
             variable_found = VariableFoundFromToken(
-                completion_context, assign_node_info.token, rep
+                completion_context,
+                assign_node_info.token,
+                rep,
+                variable_kind=VariableKind.LOCAL_ASSIGN_VARIABLE,
+                stack=stack,
             )
             collector.on_variable(variable_found)
 
-    _collect_arguments(completion_context, stack_node, collector)
-
-    accept_sets_in = {
-        normalize_robot_name("Set Local Variable"),
-    }
+    _collect_arguments(completion_context, stack, stack_node, collector)
 
     _collect_variables_from_set_keywords(
-        stack_node, completion_context, collector, accept_sets_in
+        stack_node, completion_context, collector, KEYWORD_SET_LOCAL_TO_VAR_KIND
     )
 
 
