@@ -1,6 +1,6 @@
 import os
 from functools import lru_cache
-from typing import Optional, Union, Tuple, List, Iterator
+from typing import Optional, Tuple, List, Iterator
 from robotframework_ls.impl.protocols import (
     IRobotVariableMatch,
     IRobotToken,
@@ -11,10 +11,58 @@ from robotframework_ls.impl.protocols import (
 from robocorp_ls_core.protocols import Sentinel
 from robocorp_ls_core.robotframework_log import get_logger
 import threading
+import re
 
 log = get_logger(__name__)
 
 
+def is_number_var(normalized_variable_name):
+    # see: robot.variables.finders.NumberFinder
+    try:
+        bases = {"0b": 2, "0o": 8, "0x": 16}
+        if normalized_variable_name.startswith(tuple(bases)):
+            int(normalized_variable_name[2:], bases[normalized_variable_name[:2]])
+            return True
+        int(normalized_variable_name)
+        return True
+    except:
+        pass  # Let's try float...
+
+    try:
+        float(normalized_variable_name)
+        return True
+    except:
+        pass
+
+    return False
+
+
+def is_python_eval_var(normalized_variable_name):
+    return (
+        len(normalized_variable_name) >= 2
+        and normalized_variable_name[0] == "{"
+        and normalized_variable_name[-1] == "}"
+    )
+
+
+_match_extended = re.compile(
+    r"""
+    (.+?)          # base name (group 1)
+    ([^\s\w].+)    # extended part (group 2)
+""",
+    re.UNICODE | re.VERBOSE,
+).match
+
+
+def extract_var_name_from_extended_base_name(normalized_variable_name):
+    m = _match_extended(normalized_variable_name)
+    if m is None:
+        return normalized_variable_name
+
+    base_name, _extended = m.groups()
+    return base_name
+
+
 def extract_variable_base(text: str) -> str:
     """
     Converts something as: "${S_ome.VAR}[foo]" to "S_ome.VAR".
@@ -26,7 +74,11 @@ def extract_variable_base(text: str) -> str:
             return base
 
     if len(text) >= 3:
-        if text.endswith("}") and text[1] == "{":
+        if (
+            text.endswith("}")
+            and text[1] == "{"
+            and text.startswith(("$", "@", "&", "%"))
+        ):
             return text[2:-1]
     return text
 
@@ -57,6 +109,13 @@ def robot_search_variable(text: str) -> Optional[IRobotVariableMatch]:
     return None
 
 
+def has_variable(text: str) -> bool:
+    robot_match = robot_search_variable(text)
+    if robot_match is None:
+        return False
+    return bool(robot_match.base)
+
+
 def iter_robot_variable_matches(
     string: str,
 ) -> Iterator[Tuple[IRobotVariableMatch, int]]:
@@ -93,16 +152,18 @@ def _find_split_index(string: str, eq_i: int) -> int:
         if not variables and "\\" not in string:
             return eq_i
 
+        relative_index = 0
+        robot_match_len = 0
         for robot_match, relative_index in variables:
-            before, string = (
-                robot_match.before,
-                robot_match.after,
-            )
+            before = robot_match.before
+            string = robot_match.after
             try:
                 return _find_split_index_from_part(before) + relative_index
             except ValueError:
                 pass
-        return _find_split_index_from_part(string) + relative_index
+            robot_match_len = robot_match.end - robot_match.start + len(before)
+
+        return _find_split_index_from_part(string) + relative_index + robot_match_len
     except ValueError:
         return -1
 
