@@ -1,5 +1,9 @@
 from robocorp_ls_core.python_ls import PythonLanguageServer
-from robocorp_ls_core.basic import overrides, log_and_silence_errors
+from robocorp_ls_core.basic import (
+    overrides,
+    log_and_silence_errors,
+    log_but_dont_silence_errors,
+)
 import os
 import time
 from robotframework_ls.constants import DEFAULT_COMPLETIONS_TIMEOUT
@@ -30,12 +34,16 @@ from robocorp_ls_core.lsp import (
     TextDocumentPositionParamsTypedDict,
     PositionTypedDict,
     CompletionItemTypedDict,
+    RenameParamsTypedDict,
+    PrepareRenameParamsTypedDict,
 )
 from robotframework_ls.commands import (
     ROBOT_GET_RFLS_HOME_DIR,
     ROBOT_START_INDEXING_INTERNAL,
     ROBOT_WAIT_FULL_TEST_COLLECTION_INTERNAL,
+    ROBOT_RF_INFO_INTERNAL,
 )
+from robocorp_ls_core.jsonrpc.exceptions import JsonRpcException
 
 
 log = get_logger(__name__)
@@ -363,7 +371,9 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
             },
             "hoverProvider": True,
             "referencesProvider": True,
-            "renameProvider": False,
+            "renameProvider": {
+                "prepareProvider": True,
+            },
             "foldingRangeProvider": True,
             # Note that there are no auto-trigger characters (there's no good
             # character as there's no `(` for parameters and putting it as a
@@ -500,6 +510,34 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
 
         return robot_config.get_robotframework_ls_home()
 
+    @command_dispatcher(ROBOT_RF_INFO_INTERNAL)
+    def _get_rf_info_internal(self, *arguments):
+        doc_uri = arguments[0]["uri"]
+
+        api = self._server_manager._get_others_api(doc_uri)
+        if api is not None:
+            info = api.get_interpreter_info()
+            rf_api_client = api.get_robotframework_api_client()
+            if rf_api_client is not None:
+
+                def func(monitor):
+                    ret = self._threaded_api_request(
+                        rf_api_client,
+                        "request_rf_info",
+                        doc_uri=doc_uri,
+                        monitor=monitor,
+                    )
+                    if info is not None:
+                        if isinstance(ret, dict):
+                            ret["interpreter_id"] = info.get_interpreter_id()
+                    return ret
+
+                func = require_monitor(func)
+                return func
+
+        log.info("Unable to get RF info (no api available).")
+        return None
+
     @command_dispatcher("robot.listTests")
     def _list_tests(self, *arguments):
         doc_uri = arguments[0]["uri"]
@@ -670,7 +708,7 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
             doc_uri, line, col
         )
 
-    @log_and_silence_errors(log)
+    @log_but_dont_silence_errors(log)
     def _threaded_api_request(
         self,
         rf_api_client: IRobotFrameworkApiClient,
@@ -719,9 +757,13 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
                 if result:
                     return result
 
+                error = msg.get("error")
+                if error:
+                    raise JsonRpcException(**error)
+
         return None
 
-    @log_and_silence_errors(log)
+    @log_but_dont_silence_errors(log)
     def _threaded_api_request_no_doc(
         self,
         rf_api_client: IRobotFrameworkApiClient,
@@ -805,6 +847,27 @@ class RobotFrameworkLanguageServer(PythonLanguageServer):
             "request_find_definition", "api", doc_uri, line=line, col=col
         )
 
+    def m_text_document__rename(self, **kwargs):
+        rename_params: RenameParamsTypedDict = kwargs
+        # Note: 0-based
+        doc_uri: str = rename_params["textDocument"]["uri"]
+        line: int = rename_params["position"]["line"]
+        col: int = rename_params["position"]["character"]
+        new_name = rename_params["newName"]
+        return self.async_api_forward(
+            "request_rename", "api", doc_uri, line=line, col=col, new_name=new_name
+        )
+
+    def m_text_document__prepare_rename(self, **kwargs):
+        rename_params: PrepareRenameParamsTypedDict = kwargs
+        # Note: 0-based
+        doc_uri: str = rename_params["textDocument"]["uri"]
+        line: int = rename_params["position"]["line"]
+        col: int = rename_params["position"]["character"]
+        return self.async_api_forward(
+            "request_prepare_rename", "api", doc_uri, line=line, col=col
+        )
+
     def m_text_document__signature_help(self, **kwargs):
         """
         "params": {
