@@ -263,6 +263,28 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             self._build_msg("findDefinition", doc_uri=doc_uri, line=line, col=col)
         )
 
+    def request_rename(
+        self, doc_uri: str, line: int, col: int, new_name: str
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+        return self.request_async(
+            self._build_msg(
+                "rename", doc_uri=doc_uri, line=line, col=col, new_name=new_name
+            )
+        )
+
+    def request_prepare_rename(
+        self, doc_uri: str, line: int, col: int
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+        return self.request_async(
+            self._build_msg("prepareRename", doc_uri=doc_uri, line=line, col=col)
+        )
+
     def request_source_format(
         self, text_document, options
     ) -> Optional[IIdMessageMatcher]:
@@ -329,6 +351,12 @@ class RobotFrameworkApiClient(LanguageServerClientBase):
             self._build_msg("evaluatableExpression", doc_uri=doc_uri, position=position)
         )
 
+    def request_rf_info(self, doc_uri) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+        return self.request_async(self._build_msg("rfInfo", doc_uri=doc_uri))
+
     def request_hover(
         self, doc_uri: str, line: int, col: int
     ) -> Optional[IIdMessageMatcher]:
