@@ -1,7 +1,6 @@
 from robocop import checkers
-from robocop.run import run_robocop, Robocop
 from robocop.config import Config
+from robocop.run import Robocop, run_robocop
 from robocop.version import __version__
 
-
 __all__ = ["__version__", "run_robocop", "checkers", "Robocop", "Config"]
