@@ -329,6 +329,20 @@ class IRobotFrameworkApiClient(ILanguageServerClientBase, Protocol):
         :Note: async complete.
         """
 
+    def request_rename(
+        self, doc_uri: str, line: int, col: int, new_name: str
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
+    def request_prepare_rename(
+        self, doc_uri: str, line: int, col: int
+    ) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
     def request_source_format(
         self, text_document, options
     ) -> Optional[IIdMessageMatcher]:
@@ -384,6 +398,11 @@ class IRobotFrameworkApiClient(ILanguageServerClientBase, Protocol):
         :Note: async complete.
         """
 
+    def request_rf_info(self, doc_uri: str) -> Optional[IIdMessageMatcher]:
+        """
+        :Note: async complete.
+        """
+
 
 class ITestInfoTypedDict(TypedDict):
     uri: str
@@ -512,6 +531,12 @@ class ILanguageServerClient(ILanguageServerClientBase, Protocol):
     def request_document_symbol(self, uri: str):
         pass
 
+    def request_rename(self, uri: str, line: int, col: int, new_name: str):
+        pass
+
+    def request_prepare_rename(self, uri: str, line: int, col: int):
+        pass
+
     def request_workspace_symbols(self, query: Optional[str] = None):
         pass
 
